@@ -30,6 +30,7 @@ THE SOFTWARE.  */
 #include <htslib/vcf.h>
 #include <math.h>
 
+#define FT_TAB_TEXT 0       // custom tab-delimited text file
 #define FT_GZ 1
 #define FT_VCF 2
 #define FT_VCF_GZ (FT_GZ|FT_VCF)
@@ -62,6 +63,12 @@ static inline char gt2iupac(char a, char b)
     return iupac[(int)a][(int)b];
 }
 
+static inline char nt_to_upper(char nt)
+{
+    if ( nt < 97 ) return nt;
+    return nt - 32;
+}
+
 static inline double phred_score(double prob)
 {
     if ( prob==0 ) return 99;
