@@ -1,4 +1,4 @@
-#include "pysam.h"
+#include "samtools.pysam.h"
 
 /*  stats.c -- This is the former bamcheck integrated into samtools/htslib.
 
@@ -830,8 +830,8 @@ void collect_stats(bam1_t *bam_line, stats_t *stats)
         // reads. Mates mapped to different chromosomes have isize==0.
         int32_t isize = bam_line->core.isize;
         if ( isize<0 ) isize = -isize;
-        if ( stats->info->nisize > 0 && isize >= stats->info->nisize )
-            isize = stats->info->nisize-1;
+        if ( stats->info->nisize > 0 && isize > stats->info->nisize )
+            isize = stats->info->nisize;
         if ( isize>0 || bam_line->core.tid==bam_line->core.mtid )
         {
             int pos_fst = bam_line->core.mpos - bam_line->core.pos;
@@ -1242,7 +1242,7 @@ void init_regions(stats_t *stats, const char *file)
         if ( tid < 0 )
         {
             if ( !warned )
-                fprintf(pysam_stderr,"Warning: Some sequences not present in the BAM, e.g. \"%s\". This message is printed only once.\n", line.s);
+                fprintf(samtools_stderr,"Warning: Some sequences not present in the BAM, e.g. \"%s\". This message is printed only once.\n", line.s);
             warned = 1;
             continue;
         }
@@ -1265,7 +1265,7 @@ void init_regions(stats_t *stats, const char *file)
             stats->regions[tid].pos = realloc(stats->regions[tid].pos,sizeof(pos_t)*stats->regions[tid].mpos);
         }
 
-        if ( (sscanf(&line.s[i+1],"%d %d",&stats->regions[tid].pos[npos].from,&stats->regions[tid].pos[npos].to))!=2 ) error("Could not parse the region [%s]\n", &line.s[i+1]);
+        if ( (sscanf(&line.s[i+1],"%u %u",&stats->regions[tid].pos[npos].from,&stats->regions[tid].pos[npos].to))!=2 ) error("Could not parse the region [%s]\n", &line.s[i+1]);
         if ( prev_tid==-1 || prev_tid!=tid )
         {
             prev_tid = tid;
@@ -1336,7 +1336,7 @@ void init_group_id(stats_t *stats, const char *id)
         {
             khiter_t k = kh_get(kh_rg, stats->rg_hash, key);
             if ( k != kh_end(stats->rg_hash) )
-                fprintf(pysam_stderr, "[init_group_id] The group ID not unique: \"%s\"\n", key);
+                fprintf(samtools_stderr, "[init_group_id] The group ID not unique: \"%s\"\n", key);
             int ret;
             k = kh_put(kh_rg, stats->rg_hash, key, &ret);
             kh_value(stats->rg_hash, k) = val;
@@ -1346,7 +1346,7 @@ void init_group_id(stats_t *stats, const char *id)
     if ( !n )
         error("The sample or read group \"%s\" not present.\n", id);
 #else
-    fprintf(pysam_stderr, "Samtools-htslib: init_group_id() header parsing not yet implemented\n");
+    fprintf(samtools_stderr, "Samtools-htslib: init_group_id() header parsing not yet implemented\n");
     abort();
 #endif
 }
@@ -1356,35 +1356,35 @@ static void error(const char *format, ...)
 {
     if ( !format )
     {
-        fprintf(pysam_stdout, "About: The program collects statistics from BAM files. The output can be visualized using plot-bamstats.\n");
-        fprintf(pysam_stdout, "Usage: samtools stats [OPTIONS] file.bam\n");
-        fprintf(pysam_stdout, "       samtools stats [OPTIONS] file.bam chr:from-to\n");
-        fprintf(pysam_stdout, "Options:\n");
-        fprintf(pysam_stdout, "    -c, --coverage <int>,<int>,<int>    Coverage distribution min,max,step [1,1000,1]\n");
-        fprintf(pysam_stdout, "    -d, --remove-dups                   Exclude from statistics reads marked as duplicates\n");
-        fprintf(pysam_stdout, "    -f, --required-flag  <str|int>      Required flag, 0 for unset. See also `samtools flags` [0]\n");
-        fprintf(pysam_stdout, "    -F, --filtering-flag <str|int>      Filtering flag, 0 for unset. See also `samtools flags` [0]\n");
-        fprintf(pysam_stdout, "        --GC-depth <float>              the size of GC-depth bins (decreasing bin size increases memory requirement) [2e4]\n");
-        fprintf(pysam_stdout, "    -h, --help                          This help message\n");
-        fprintf(pysam_stdout, "    -i, --insert-size <int>             Maximum insert size [8000]\n");
-        fprintf(pysam_stdout, "    -I, --id <string>                   Include only listed read group or sample name\n");
-        fprintf(pysam_stdout, "    -l, --read-length <int>             Include in the statistics only reads with the given read length []\n");
-        fprintf(pysam_stdout, "    -m, --most-inserts <float>          Report only the main part of inserts [0.99]\n");
-        fprintf(pysam_stdout, "    -P, --split-prefix <str>            Path or string prefix for filepaths output by -S (default is input filename)\n");
-        fprintf(pysam_stdout, "    -q, --trim-quality <int>            The BWA trimming parameter [0]\n");
-        fprintf(pysam_stdout, "    -r, --ref-seq <file>                Reference sequence (required for GC-depth and mismatches-per-cycle calculation).\n");
-        fprintf(pysam_stdout, "    -s, --sam                           Ignored (input format is auto-detected).\n");
-        fprintf(pysam_stdout, "    -S, --split <tag>                   Also write statistics to separate files split by tagged field.\n");
-        fprintf(pysam_stdout, "    -t, --target-regions <file>         Do stats in these regions only. Tab-delimited file chr,from,to, 1-based, inclusive.\n");
-        fprintf(pysam_stdout, "    -x, --sparse                        Suppress outputting IS rows where there are no insertions.\n");
-        sam_global_opt_help(pysam_stdout, "-.--.");
-        fprintf(pysam_stdout, "\n");
+        fprintf(samtools_stdout, "About: The program collects statistics from BAM files. The output can be visualized using plot-bamstats.\n");
+        fprintf(samtools_stdout, "Usage: samtools stats [OPTIONS] file.bam\n");
+        fprintf(samtools_stdout, "       samtools stats [OPTIONS] file.bam chr:from-to\n");
+        fprintf(samtools_stdout, "Options:\n");
+        fprintf(samtools_stdout, "    -c, --coverage <int>,<int>,<int>    Coverage distribution min,max,step [1,1000,1]\n");
+        fprintf(samtools_stdout, "    -d, --remove-dups                   Exclude from statistics reads marked as duplicates\n");
+        fprintf(samtools_stdout, "    -f, --required-flag  <str|int>      Required flag, 0 for unset. See also `samtools flags` [0]\n");
+        fprintf(samtools_stdout, "    -F, --filtering-flag <str|int>      Filtering flag, 0 for unset. See also `samtools flags` [0]\n");
+        fprintf(samtools_stdout, "        --GC-depth <float>              the size of GC-depth bins (decreasing bin size increases memory requirement) [2e4]\n");
+        fprintf(samtools_stdout, "    -h, --help                          This help message\n");
+        fprintf(samtools_stdout, "    -i, --insert-size <int>             Maximum insert size [8000]\n");
+        fprintf(samtools_stdout, "    -I, --id <string>                   Include only listed read group or sample name\n");
+        fprintf(samtools_stdout, "    -l, --read-length <int>             Include in the statistics only reads with the given read length []\n");
+        fprintf(samtools_stdout, "    -m, --most-inserts <float>          Report only the main part of inserts [0.99]\n");
+        fprintf(samtools_stdout, "    -P, --split-prefix <str>            Path or string prefix for filepaths output by -S (default is input filename)\n");
+        fprintf(samtools_stdout, "    -q, --trim-quality <int>            The BWA trimming parameter [0]\n");
+        fprintf(samtools_stdout, "    -r, --ref-seq <file>                Reference sequence (required for GC-depth and mismatches-per-cycle calculation).\n");
+        fprintf(samtools_stdout, "    -s, --sam                           Ignored (input format is auto-detected).\n");
+        fprintf(samtools_stdout, "    -S, --split <tag>                   Also write statistics to separate files split by tagged field.\n");
+        fprintf(samtools_stdout, "    -t, --target-regions <file>         Do stats in these regions only. Tab-delimited file chr,from,to, 1-based, inclusive.\n");
+        fprintf(samtools_stdout, "    -x, --sparse                        Suppress outputting IS rows where there are no insertions.\n");
+        sam_global_opt_help(samtools_stdout, "-.--.@");
+        fprintf(samtools_stdout, "\n");
     }
     else
     {
         va_list ap;
         va_start(ap, format);
-        vfprintf(pysam_stderr, format, ap);
+        vfprintf(samtools_stderr, format, ap);
         va_end(ap);
     }
     exit(1);
@@ -1483,13 +1483,13 @@ int init_stat_info_fname(stats_info_t* info, const char* bam_fname, const htsFor
     // .. bam
     samFile* sam;
     if ((sam = sam_open_format(bam_fname, "r", in_fmt)) == 0) {
-        error("Failed to open: %s\n", bam_fname);
+        print_error_errno("stats", "failed to open \"%s\"", bam_fname);
         return 1;
     }
     info->sam = sam;
     info->sam_header = sam_hdr_read(sam);
     if (info->sam_header == NULL) {
-        error("Failed to read header for '%s'\n", bam_fname);
+        print_error("stats", "failed to read header for \"%s\"", bam_fname);
         return 1;
     }
     return 0;
@@ -1539,7 +1539,7 @@ static void init_stat_structs(stats_t* stats, stats_info_t* info, const char* gr
     stats->quals_2nd      = calloc(stats->nquals*stats->nbases,sizeof(uint64_t));
     stats->gc_1st         = calloc(stats->ngc,sizeof(uint64_t));
     stats->gc_2nd         = calloc(stats->ngc,sizeof(uint64_t));
-    stats->isize          = init_isize_t(info->nisize);
+    stats->isize          = init_isize_t(info->nisize ?info->nisize+1 :0);
     stats->gcd            = calloc(stats->ngcd,sizeof(gc_depth_t));
     stats->mpc_buf        = info->fai ? calloc(stats->nquals*stats->nbases,sizeof(uint64_t)) : NULL;
     stats->acgtno_cycles  = calloc(stats->nbases,sizeof(acgtno_count_t));
@@ -1598,7 +1598,7 @@ int main_stats(int argc, char *argv[])
 
     static const struct option loptions[] =
     {
-        SAM_OPT_GLOBAL_OPTIONS('-', 0, '-', '-', 0),
+        SAM_OPT_GLOBAL_OPTIONS('-', 0, '-', '-', 0, '@'),
         {"help", no_argument, NULL, 'h'},
         {"remove-dups", no_argument, NULL, 'd'},
         {"sam", no_argument, NULL, 's'},
@@ -1620,7 +1620,7 @@ int main_stats(int argc, char *argv[])
     };
     int opt;
 
-    while ( (opt=getopt_long(argc,argv,"?hdsxr:c:l:i:t:m:q:f:F:I:1:S:P:",loptions,NULL))>0 )
+    while ( (opt=getopt_long(argc,argv,"?hdsxr:c:l:i:t:m:q:f:F:I:1:S:P:@:",loptions,NULL))>0 )
     {
         switch (opt)
         {
@@ -1664,6 +1664,8 @@ int main_stats(int argc, char *argv[])
     }
 
     if (init_stat_info_fname(info, bam_fname, &ga.in)) return 1;
+    if (ga.nthreads > 0)
+        hts_set_threads(info->sam, ga.nthreads);
 
     stats_t *all_stats = stats_init();
     stats_t *curr_stats = NULL;
@@ -1710,13 +1712,13 @@ int main_stats(int argc, char *argv[])
         }
 
         if (ret < -1) {
-            fprintf(pysam_stderr, "Failure while decoding file\n");
+            fprintf(samtools_stderr, "Failure while decoding file\n");
             return 1;
         }
     }
 
     round_buffer_flush(all_stats, -1);
-    output_stats(pysam_stdout, all_stats, sparse);
+    output_stats(samtools_stdout, all_stats, sparse);
     if (info->split_tag)
         output_split_stats(split_hash, bam_fname, sparse);
 
